// 
// JetHott
// Debimetre carburant pour Jet
// pour telemesure Graupner Hott
// 
// Version 0.24
// avril 2019
//
// Auteur : www.thyzoon.fr 
// 2011-2019 
//

// -------------------------------------------------------------- 
// Caractéristiques :
// - Mesure de 3 batteries LiPo 2S
// - Mesure de debit carburant sur capteur BioTech FCH-M-PP  50ml - 800ml
// --------------------------------------------------------------

// --------------------------------------------------------------
// Interfaces ARDUINO :
//
// A0        : Lipo 1
// A1        : Lipo 2
// A2        : Lipo 3
// INT0 (D2) : Signal debitmetre
// TX / RX   : COM Hott
//
    
#include <EEPROM.h>    
#include "Message.h"
#include "Tension_Lipo.h"

// Def LED verte carte Arduino mini pro
#define LEDPIN_PINMODE          pinMode (13, OUTPUT);
#define LEDPIN_SWITCH           PINB |= 1<<5;     //switch LEDPIN state (digital PIN 13)
#define LEDPIN_OFF              PORTB &= ~(1<<5);
#define LEDPIN_ON               PORTB |= (1<<5);

// Baudrate UART COM Hott
#define SERIAL_COM_SPEED 		19200

// Code son des alarmes (et messages vocaux associés)
#define ALARME_CARBURANT        0x15  // code son Hott / Message vocal 
#define ALARME_TEMPERATURE      0x08  // code son Hott / Message vocal
#define ALARME_POWER_MIN        0x10  // code son Hott / Message vocal       Min. Power Voltage
#define ALARME_TENSION_1_MIN    0x12  // code son Hott / Message vocal
#define ALARME_TENSION_2_MIN    0x13  // code son Hott / Message vocal
#define ALARME_RPM_MAXI         0x19  // code son Hott / Message vocal

// 
GMessage message;
TensionLipo tension;

// ---------------
// Fonctions init interruptions
void Init_Int_Debit();  // Initialisation des interruptions

// Variables LiPo
float lipo1 = 0.0; // Tension LiPo 1
float lipo2 = 0.0;
float lipo3 = 0.0;

byte bip_alarme;                     // son de l'alarme Hott pour émission

// Accès aux paramètres lus en EEPROM dans message.cpp
extern word seuil_tension_alarme;       // seuil tension alarme 
extern word temps_repetition;           // Repetition alarme (temps entre alarme)
extern word alarme_lipo_2;              // On/Off alarme pour LiPo 2
extern word alarme_lipo_3;              // On/Off alarme pour LiPo 3
extern word impulsions_litre;           // nb impulsion / litre du debitmetre (valeur d'origine ou corrigee)
extern word capa_reservoir;             // Capacite du reservoir
extern word seuil_reservoir;            // Seuil alarme carburant
extern word alarme_carburant;           // On/Off alarme carburant
extern word mode_gst_carburant;         // Mode de gestion du carburant
extern word qt_carburant;               // Quantite de carburant a consommer

word volume_carburant;                  // Volume actuel de carburant dans le reservoir (en ml)

unsigned long temps_courant;            // temp en ms
unsigned long temps_prochaine_alarme;   // temps pour la prochaine alarme, s'il y a un defaut

// Gestion du debitmetre 
unsigned long compteur_pulse_debit;     // Compteur impulsion
boolean flag_pulse_debit;               // Flag d'interruption
//unsigned long delta_temps;            // Delta t entre 2 interruptions en µs

float imp_ml;                           // nb d'impulsion par ml
float conso_carburant;                  // carburant consomme en ml 

// Init
void setup() {
  // Init communications 
  message.init();

  tension.begin ();
  
  volume_carburant = capa_reservoir;     // Au demarrage, on suppose que le reservoir est plein !
  conso_carburant = 0.0;                 // Carburant consomme, = rien ...
  imp_ml = float(impulsions_litre) / 1000.0;  // Nb d'impulsions / ml du debitmetre
  
  bip_alarme = 0;                // pas de son alarme
  temps_prochaine_alarme = 0;

  compteur_pulse_debit = 0;   // Nb impulsions debitmetre
  flag_pulse_debit = 0;       // Presence interruption debitmetre
  // delta_temps = 0;            // Temps entre 2 INT 
  
	Init_Int_Debit();     // Initialisation interruptions signal debitmetre
	
  LEDPIN_PINMODE
  LEDPIN_OFF 
}

// Boucle pricipale
void loop() {
	
  // Gestion des messages	Hott
  message.main();

  // Mesure des 3 tensions LiPo 
  lipo2 = tension.Tension_Lipo1();  
  lipo1 = tension.Tension_Lipo2();  
  lipo3 = tension.Tension_Lipo3();    
  
  // Si interruption par Pulse Debitmetre
  if (flag_pulse_debit) {
    // Caclul du carburant consomme
    conso_carburant = float (compteur_pulse_debit) / imp_ml;   // Carburant consomme en ml
    // Volume dans le reservoir
    volume_carburant = capa_reservoir - word(conso_carburant);
  }
 
  // Gestion des alarmes (par la telemesure Hott)
  // La variable utilisee est : bip_alarme
  // Si  (bip_alarme = 0) , il n'y a pas d'alarme de transmise
  // Sinon (bip_alarme = au code son) que l'on veut emettre (type d'alarme, message associé predefini Hott)
  
  // Les radios Graupner MX-xx ne traitent qu'une alarme vocale à la fois,
  // Si l'on envoie plusieurs alarmes à la suite sans attendre la
  // fin du message vocal en cours, les alarmes suivantes sont perdues.
	
  // Gestion du temps entre alarme afin de laisser assez de temps au message 
  // vocal d'être émis complètement avant envoi de l'alarme suivante
  temps_courant = millis();  // temps courant ms (Temps Arduino depuis demarrage)
  
  // Si pas d'alarme en attente d'emission et que le temps est > au temps requis ...
  // Les temps sont en ms
  if ((bip_alarme == 0) && (temps_courant > temps_prochaine_alarme)) {
	  // Attention, la priorite est traitee par l'ordre des 4 'if' qui suivent, le dernier etant le + prioritaire
    // S'il y a plusieurs alarme, une seule alarme sera traitee !   
    if ((lipo3 < ((float) seuil_tension_alarme / 100.0)) && (alarme_lipo_3 == 1))    // Si alarme activée sur Bat 3 dans le menu radio JetHott
		  bip_alarme = ALARME_POWER_MIN;       
	
    if ((lipo2 < ((float) seuil_tension_alarme / 100.0)) && (alarme_lipo_2 == 1))    // Si alarme activée sur Bat 2 dans le menu radio JetHott
		  bip_alarme = ALARME_TENSION_2_MIN;  
  
    if (lipo1 < ((float) seuil_tension_alarme / 100.0))
	    bip_alarme = ALARME_TENSION_1_MIN;

	  //+++++++++++++++++++++++++++++++++++++++++++++++++++++
    //  ALARME PRIORITAIRE
	  switch (mode_gst_carburant) {
		  case 0 :   // Mode Reservoir
           if ((volume_carburant <= seuil_reservoir) && (alarme_carburant == 1)) {   // Si alarme activée dans le menu radio JetHott
              bip_alarme = ALARME_CARBURANT;
              LEDPIN_ON 
           }   
		       break; 	  
	    case 1 :   // Mode Quantite
		       if ((word(conso_carburant) >= qt_carburant) && (alarme_carburant == 1)) {   // Si alarme activée dans le menu radio JetHott
              bip_alarme = ALARME_CARBURANT;
              LEDPIN_ON 
		       }   
		       break; 
        default :
		       break; 
	  } // End switch (mode_gst_carburant)		   

    // Dans le cas de presence d'une alarme, mise a jour du temps de la prochaine alarme selon reglage menu parametres
    if (bip_alarme != 0)
      temps_prochaine_alarme = temps_courant + (temps_repetition*1000);  // Calcul du temps de la prochaine alarme

  } // End if ((bip_alarme .....
  
}
